@@ -60,13 +60,14 @@ namespace Lum {
         positionOS
       };
 
-      class LUMAPI RedrawMsg : public ::Lum::Base::ResyncMsg
+      class LUMAPI DrawMsg : public ::Lum::Base::ResyncMsg
       {
       public:
-        int    x;
-        int    y;
-        size_t width;
-        size_t height;
+        DrawInfo* draw;
+        int       x;
+        int       y;
+        size_t    width;
+        size_t    height;
       };
 
       class LUMAPI ResizeMsg : public ::Lum::Base::ResyncMsg
@@ -161,7 +162,7 @@ namespace Lum {
       virtual Model::Action* GetMouseInAction() const = 0;
       virtual Model::Action* GetMouseOutAction() const = 0;
       virtual Model::Action* GetResizeAction() const = 0;
-      virtual Model::Action* GetRedrawAction() const = 0;
+      virtual Model::Action* GetDrawAction() const = 0;
       virtual Model::Action* GetPreInitAction() const = 0;
       virtual Model::Action* GetEventAction() const = 0;
       virtual Model::Action* GetGrabCancelAction() const =0;
@@ -221,21 +222,10 @@ namespace Lum {
       /**
         @name Double buffering methods
       */
-      //@{
-      /**
-        On the first call of a pair: Exchange the current draw info for the visible
-        window contents with an alternative DrawInfo structure for an off-screen
-        background buffer. You can draw into this buffer without these drawing
-        getting visible. Calling SwitchBackgroundDrawInfo() again
-        will then make this changes visible.
-      */
-      virtual bool SwitchBackgroundDrawInfo(int x, int y, size_t width, size_t height) = 0;
-      //@}
+
 
       virtual Window* GetTopWindow() = 0;
 
-      virtual DrawInfo* GetDrawInfo() const = 0;
-
       virtual void Resize(size_t width, size_t height) = 0;
 
       virtual void GrabMouse(bool grab) = 0;
@@ -245,7 +235,7 @@ namespace Lum {
                                       unsigned long timeout) = 0;
 
       virtual void TriggerResize(size_t width, size_t height) = 0;
-      virtual void TriggerRedraw(int x, int y, size_t width, size_t height) = 0;
+      virtual void TriggerDraw(int x, int y, size_t width, size_t height) = 0;
     };
   }
 }
