@@ -23,7 +23,7 @@
 #include <Lum/Base/Object.h>
 
 #include <Lum/Model/Boolean.h>
-#include <Lum/Model/Integer.h>
+#include <Lum/Model/Number.h>
 
 #include <Lum/OS/DrawInfo.h>
 #include <Lum/OS/Image.h>
@@ -49,7 +49,8 @@ namespace Lum {
     bool IsIn(const OS::MouseEvent& event) const;
 
   protected:
-    virtual void DrawContent(int x, int y, size_t w, size_t h) = 0;
+    virtual void DrawContent(OS::DrawInfo* draw,
+                             int x, int y, size_t w, size_t h) = 0;
 
   public:
     State(bool active=true);
@@ -63,7 +64,8 @@ namespace Lum {
     bool HandleKeyEvent(const OS::KeyEvent& event);
 
     void PrepareForBackground(OS::DrawInfo* draw);
-    void Draw(int x, int y, size_t w, size_t h);
+    void Draw(OS::DrawInfo* draw,
+              int x, int y, size_t w, size_t h);
 
     void Resync(Base::Model* model, const Base::ResyncMsg& msg);
   };
@@ -74,7 +76,8 @@ namespace Lum {
     OS::ImageRef image;
 
   protected:
-    void DrawContent(int x, int y, size_t w, size_t h);
+    void DrawContent(OS::DrawInfo* draw,
+                     int x, int y, size_t w, size_t h);
 
   public:
     ImageState(OS::Image* image, bool active=true);
