@@ -130,7 +130,8 @@ namespace Lum {
 
     void CalcSize();
 
-    void Draw(int x, int y, size_t w, size_t h);
+    void Draw(OS::DrawInfo* draw,
+              int x, int y, size_t w, size_t h);
 
     static Text* Create(const std::wstring& text,
                         bool horizontalFlex=false, bool verticalFlex=false);
