@@ -23,7 +23,7 @@
 #include <cmath>
 
 #include <Lum/Base/Util.h>
-#include <iostream>
+
 #define MOVE_TIMER_MILLISECONDS      300
 #define KINETIC_TIMER_MILLISECONDS    50
 #define KINETIC_MIN_SPEED          100.0
@@ -92,7 +92,7 @@ namespace Lum {
 
       diff.Sub(lastMoveX);
 
-      unsigned long diffMillis=diff.GetTime()*1000+diff.GetMicroseconds()/1000;
+      long diffMillis=diff.GetTime()*1000+diff.GetMicroseconds()/1000;
 
       speedX=0;
 
@@ -117,7 +117,7 @@ namespace Lum {
 
       diff.Sub(lastMoveY);
 
-      unsigned long diffMillis=diff.GetTime()*1000+diff.GetMicroseconds()/1000;
+      long diffMillis=diff.GetTime()*1000+diff.GetMicroseconds()/1000;
 
       speedY=0;
 
@@ -265,16 +265,8 @@ namespace Lum {
           posY=scrollable->GetVAdjustment()->GetTotal()-scrollable->GetVAdjustment()->GetVisible()+1;
         }
 
-        scrollable->GetHAdjustment()->Off();
-        scrollable->GetVAdjustment()->Off();
-
         scrollable->GetHAdjustment()->SetTop(posX);
         scrollable->GetVAdjustment()->SetTop(posY);
-
-        scroller->Redraw();
-
-        scrollable->GetHAdjustment()->On(false);
-        scrollable->GetVAdjustment()->On(false);
       }
 
       return true;
@@ -307,9 +299,6 @@ namespace Lum {
       int pixelX=(int)rint(speedX*KINETIC_TIMER_MILLISECONDS/1000);
       int pixelY=(int)rint(speedY*KINETIC_TIMER_MILLISECONDS/1000);
 
-      scrollable->GetHAdjustment()->Off();
-      scrollable->GetVAdjustment()->Off();
-
       if (pixelX>0) {
         scrollable->GetHAdjustment()->IncTop((size_t)pixelX);
       }
@@ -324,11 +313,6 @@ namespace Lum {
         scrollable->GetVAdjustment()->DecTop((size_t)std::abs(pixelY));
       }
 
-      scroller->Redraw();
-
-      scrollable->GetHAdjustment()->On(false);
-      scrollable->GetVAdjustment()->On(false);
-
       if (std::abs(speedX)<1 && std::abs(speedY)<1) {
         state=pending;
       }
@@ -341,7 +325,9 @@ namespace Lum {
     }
     else if (model==scrollable->GetHAdjustment()->GetTopModel() ||
              model==scrollable->GetVAdjustment()->GetTopModel()) {
-      state=pending;
+      if (state==scrollMaybe) {
+        state=pending;
+      }
     }
   }
 }
