@@ -21,7 +21,6 @@
 
 #include <Lum/Base/Util.h>
 
-#include <Lum/OS/Behaviour.h>
 #include <Lum/OS/Display.h>
 
 namespace Lum {
@@ -218,9 +217,10 @@ namespace Lum {
     return false;
   }
 
-  void Knob::Draw(int x, int y, size_t w, size_t h)
+  void Knob::Draw(OS::DrawInfo* draw,
+                  int x, int y, size_t w, size_t h)
   {
-    Control::Draw(x,y,w,h);
+    Control::Draw(draw,x,y,w,h);
 
     if (!OIntersect(x,y,w,h)) {
       return;
@@ -236,7 +236,6 @@ namespace Lum {
 
     size_t       kSize,bSize;
     int          kStart;
-    OS::DrawInfo *draw=GetDrawInfo();
 
     if (IsMouseActive()) {
       draw->activated=true;
@@ -285,7 +284,7 @@ namespace Lum {
     }
 
     draw->selected=selected;
-    knob->Draw(knob->GetOX(),knob->GetOY(),knob->GetOWidth(),knob->GetOHeight());
+    knob->Draw(draw,knob->GetOX(),knob->GetOY(),knob->GetOWidth(),knob->GetOHeight());
     draw->selected=false;
 
     draw->activated=false;
@@ -311,17 +310,22 @@ namespace Lum {
 
   void Knob::Resync(Base::Model* model, const Base::ResyncMsg& msg)
   {
-    if (model==incAction && incAction->IsFinished()) {
+    if (model==incAction &&
+        incAction->IsFinished()) {
       if (adjustment.Valid()) {
         adjustment->IncTop();
       }
     }
-    else if (model==decAction && decAction->IsFinished()) {
+    else if (model==decAction &&
+             decAction->IsFinished()) {
       if (adjustment.Valid()) {
         adjustment->DecTop();
       }
     }
-    else if (model==adjustment && visible && adjustment.Valid() && adjustment->IsEnabled()) {
+    else if (model==adjustment &&
+             visible &&
+             adjustment.Valid() &&
+             adjustment->IsEnabled()) {
       Redraw();
     }
   }
