@@ -24,6 +24,33 @@
 namespace Lum {
   namespace Model {
 
+    void String::SetMaxLength(size_t maxLength)
+    {
+      this->maxLength=maxLength;
+
+      if (!IsNull() &&
+          value.value.length()>maxLength) {
+        SetNull();
+      }
+
+      this->Notify();
+    }
+
+    size_t String::GetMaxLength() const
+    {
+      return maxLength;
+    }
+
+    void String::Set(const std::wstring& value)
+    {
+      if (value.length()<=maxLength) {
+        Simple<std::wstring>::Set(value);
+      }
+      else {
+        SetNull();
+      }
+    }
+
     void String::Erase(size_t start, size_t length)
     {
       assert(!IsNull());
@@ -37,6 +64,10 @@ namespace Lum {
     {
       assert(!IsNull());
 
+      if (value.value.length()+text.length()>maxLength) {
+        return;
+      }
+
       value.value.insert(pos,text);
 
       Notify();
@@ -51,6 +82,5 @@ namespace Lum {
     {
       return value.value.empty();
     }
-
   }
 }
