@@ -38,6 +38,8 @@
 
 #include <Lum/Base/String.h>
 
+#include <Lum/Manager/Display.h>
+
 #include <Lum/OS/X11/Display.h>
 
 namespace Lum {
@@ -152,9 +154,10 @@ namespace Lum {
         balloonId(0),
         window(0),fullScreen(false),
         customWindowType(0),
+        checkRefreshAction(new Model::Action()),
         xic(0)
       {
-        // no code
+        Observe(checkRefreshAction);
       }
 
       Window::~Window()
@@ -162,6 +165,16 @@ namespace Lum {
         // no code
       }
 
+      void Window::Resync(Lum::Base::Model* model,
+                          const Lum::Base::ResyncMsg& msg)
+      {
+        if (model==checkRefreshAction && checkRefreshAction->IsFinished()) {
+          CheckRefresh();
+        }
+
+        OS::Base::Window::Resync(model,msg);
+      }
+
       void Window::SetTitle(const std::wstring& name)
       {
         OS::Base::Window::SetTitle(name);
@@ -450,79 +463,53 @@ data        : POINTER TO ARRAY OF LONGINT;*/
             width=x11Event.xconfigure.width;
             height=x11Event.xconfigure.height;
             draw->SetNewSize(width,height);
+
             TriggerResize(x11Event.xconfigure.width-x11Event.xconfigure.border_width,
                           x11Event.xconfigure.height-x11Event.xconfigure.border_width);
-            TriggerRedraw(0,0,GetWidth(),GetHeight());
+            TriggerDraw(0,0,GetWidth(),GetHeight());
+            CheckRefresh();
           }
           return true;
         case Expose:
           {
-            int x1,y1,x2,y2;
-
-            draw->PushClipBegin(x11Event.xexpose.x,x11Event.xexpose.y,
-                                x11Event.xexpose.width,x11Event.xexpose.height);
-
-            x1=x11Event.xexpose.x;
-            y1=x11Event.xexpose.y;
-            x2=x11Event.xexpose.x+x11Event.xexpose.width-1;
-            y2=x11Event.xexpose.y+x11Event.xexpose.height-1;
-
-            while (XCheckTypedWindowEvent(display->display,window,Expose,&x11Event)==True) {
-              if (x11Event.xexpose.x<x1) {
-                x1=x11Event.xexpose.x;
-              }
-              if (x11Event.xexpose.y<y1) {
-                y1=x11Event.xexpose.y;
-              }
-              if (x11Event.xexpose.x+x11Event.xexpose.width-1>x2) {
-                x2=x11Event.xexpose.x+x11Event.xexpose.width-1;
-              }
-              if (x11Event.xexpose.y+x11Event.xexpose.height-1>y2) {
-                y2=x11Event.xexpose.y+x11Event.xexpose.height-1;
-              }
-
-              draw->AddClipRegion(x11Event.xexpose.x,x11Event.xexpose.y,
-                                  x11Event.xexpose.width,x11Event.xexpose.height);
+            if (IsShown() && !onOpened) {
+              TriggerPreOpened();
             }
 
-            draw->PushClipEnd();
-            TriggerRedraw(x1,y1,x2-x1+1,y2-y1+1);
-            draw->PopClip();
+            TriggerDraw(x11Event.xexpose.x,
+                        x11Event.xexpose.y,
+                        x11Event.xexpose.width,
+                        x11Event.xexpose.height);
+
+            while (XCheckTypedWindowEvent(display->display,window,Expose,&x11Event)==True) {
+              TriggerDraw(x11Event.xexpose.x,
+                          x11Event.xexpose.y,
+                          x11Event.xexpose.width,
+                          x11Event.xexpose.height);
+            }
+
+            CheckRefresh();
+
+            if (IsShown() && !onOpened) {
+              TriggerOpened();
+            }
           }
           return true;
         case GraphicsExpose:
           {
-            int x1,y1,x2,y2;
-
-            draw->PushClipBegin(x11Event.xgraphicsexpose.x,x11Event.xgraphicsexpose.y,
-                                x11Event.xgraphicsexpose.width,x11Event.xgraphicsexpose.height);
-
-            x1=x11Event.xgraphicsexpose.x;
-            y1=x11Event.xgraphicsexpose.y;
-            x2=x11Event.xgraphicsexpose.x+x11Event.xgraphicsexpose.width-1;
-            y2=x11Event.xgraphicsexpose.y+x11Event.xgraphicsexpose.height-1;
+            TriggerDraw(x11Event.xgraphicsexpose.x,
+                        x11Event.xgraphicsexpose.y,
+                        x11Event.xgraphicsexpose.width,
+                        x11Event.xgraphicsexpose.height);
 
             while (XCheckTypedWindowEvent(display->display,window,GraphicsExpose,&x11Event)==True) {
-              if (x11Event.xgraphicsexpose.x<x1) {
-                x1=x11Event.xgraphicsexpose.x;
-              }
-              if (x11Event.xgraphicsexpose.y<y1) {
-                y1=x11Event.xgraphicsexpose.y;
-              }
-              if (x11Event.xgraphicsexpose.x+x11Event.xgraphicsexpose.width-1>x2) {
-                x2=x11Event.xgraphicsexpose.x+x11Event.xgraphicsexpose.width-1;
-              }
-              if (x11Event.xgraphicsexpose.y+x11Event.xgraphicsexpose.height-1>y2) {
-                y2=x11Event.xgraphicsexpose.y+x11Event.xgraphicsexpose.height-1;
-              }
-
-              draw->AddClipRegion(x11Event.xgraphicsexpose.x,x11Event.xgraphicsexpose.y,
-                                  x11Event.xgraphicsexpose.width,x11Event.xgraphicsexpose.height);
+              TriggerDraw(x11Event.xgraphicsexpose.x,
+                          x11Event.xgraphicsexpose.y,
+                          x11Event.xgraphicsexpose.width,
+                          x11Event.xgraphicsexpose.height);
             }
 
-            draw->PushClipEnd();
-            TriggerRedraw(x1,y1,x2-x1+1,y2-y1+1);
-            draw->PopClip();
+            CheckRefresh();
           }
           return true;
         case CreateNotify:
@@ -658,6 +645,116 @@ data        : POINTER TO ARRAY OF LONGINT;*/
         return Base::Window::HandleEvent(event);
       }
 
+      void Window::TriggerDraw(int x, int y, size_t width, size_t height)
+      {
+
+        /*
+          Try to optimize redrawRects list
+        */
+        std::vector<Rectangle>::iterator r=redrawRects.begin();
+        while (r!=redrawRects.end()) {
+          /*
+            If the new rectangle is completly contained by an existing rectangle,
+            skip insertion.
+            */
+          if (x>=r->x &&
+              y>=r->y &&
+              x+width<=r->x+r->width &&
+              y+height<=r->y+r->height) {
+            return;
+          }
+          /*
+            If the new rectange completely contains an existing rectangle,
+            exchange data of the existing rectangle with the data of the new
+            rectangle.
+          */
+          else if (x<=r->x &&
+                   y<=r->y &&
+              x+width>=r->x+r->width &&
+              y+height>=r->y+r->height) {
+            r->x=x;
+            r->y=y;
+            r->width=width;
+            r->height=height;
+            return;
+          }
+
+          ++r;
+        }
+
+        Rectangle rect;
+
+        rect.x=x;
+        rect.y=y;
+        rect.width=width;
+        rect.height=height;
+
+        redrawRects.push_back(rect);
+
+        if (redrawRects.size()==1) {
+          display->QueueActionForEventLoop(checkRefreshAction);
+        }
+      }
+
+      bool Window::CheckRefresh()
+      {
+        if (redrawRects.size()==0) {
+          return false;
+        }
+
+        if (!IsShown()) {
+          redrawRects.clear();
+          return false;
+        }
+
+        // We do not redraw anything if the display is off...
+        if (Manager::Display::Instance()!=NULL &&
+            Manager::Display::Instance()->GetDisplayState()==Manager::Display::DisplayStateOff) {
+          redrawRects.clear();
+          return false;
+        }
+
+        int x1=redrawRects.front().x;
+        int y1=redrawRects.front().y;
+        int x2=x1+redrawRects.front().width-1;
+        int y2=y1+redrawRects.front().height-1;
+
+        for (std::vector<Rectangle>::const_iterator rect=redrawRects.begin();
+             rect!=redrawRects.end();
+             ++rect) {
+          x1=std::min(x1,rect->x);
+          y1=std::min(y1,rect->y);
+          x2=std::max(x2,(int)(rect->x+rect->width-1));
+          y2=std::max(y2,(int)(rect->y+rect->height-1));
+        }
+
+        DrawInfo* draw=RequestRefreshDrawInfo(x1,y1,x2-x1+1,y2-y1+1);
+
+        for (std::vector<Rectangle>::const_iterator rect=redrawRects.begin();
+             rect!=redrawRects.end();
+             ++rect) {
+          draw->PushClip(rect->x,rect->y,rect->width,rect->height);
+
+          OS::Window::DrawMsg msg;
+
+          msg.draw=draw;
+          msg.x=rect->x;
+          msg.y=rect->y;
+          msg.width=rect->width;
+          msg.height=rect->height;
+
+          drawAction->Trigger(msg);
+
+          draw->PopClip();
+        }
+
+        ReleaseRefreshDrawInfo(x1,y1,x2-x1+1,y2-y1+1);
+
+        redrawRects.clear();
+
+        return true;
+      }
+
       void Window::Resize(size_t width, size_t height)
       {
         Base::Window::Resize(width,height);
